#include "Thread.hh"
#include "Guard.hh"
#include "ThreadError.hh"
#include <errno.h>

#include <iostream>
using namespace std;

Thread::Thread(bool joinable) 
	: thread_id_(0),
	  is_started_(false),
	  joinable_(joinable)
{

}

Thread::~Thread() {
	// nothing to do...
}

void* Thread::internal_run(void* arg) {
	Thread* thread=static_cast<Thread*>(arg);
	thread->run();
	return NULL;
}

void Thread::start() {
	Guard guard(mutex_);

	if(is_started_){
		throw ThreadError("thread already started...");
	}

	pthread_attr_t detach_attr_;

	int res=pthread_attr_init(&detach_attr_);
	if(res!=0) {
		throw ThreadError("unsuccessful pthread_attr_init()...");
	}

	res=pthread_attr_setdetachstate(&detach_attr_,(joinable_?PTHREAD_CREATE_JOINABLE:PTHREAD_CREATE_DETACHED));
	if(res!=0) {
		throw ThreadError("unsuccessful pthread_attr_setdetachstate()...");
	}

	res=pthread_create(&thread_id_,&detach_attr_,
		Thread::internal_run,this);
	if(res!=0) {
		throw ThreadError("can't create thread... ");
	}

	res=pthread_attr_destroy(&detach_attr_);
	if(res!=0) {
		throw ThreadError("unsuccessfull pthread_attr_destroy()...");
	}

	is_started_=true;
}

bool Thread::is_started() {
	Guard guard(mutex_);
	return is_started_;
}

bool Thread::is_joinable() {
	Guard guard(mutex_);
	return joinable_;
}

void Thread::join() {
	{
		Guard guard(mutex_);
		if(!is_started_) {
			throw ThreadError("trying to join not started thread...");
		}
	
	}
	pthread_join(thread_id_,NULL);	
}



